local console = require "console"
-- added helpful nativefs library to be able to utilize folders outside of save folder
nativefs = require "nativefs"
-- Slab is Gui for texteditor
Slab = require "Slab"

OS = love.system.getOS()
W, H, Flags = nil, nil, nil
EditorY, EditorH = nil, nil

MenuY = 0.0
if OS == 'iOS' then
	MenuY = 20.0
end
MenuHeight = 20.0

function love.run()
	if love.load then
		love.load(love.arg.parseGameArguments(arg), arg)
	end

	-- We don't want the first frame's dt to include time taken by love.load.
	if love.timer then
		love.timer.step()
	end

	local dt = 0

	-- Main loop time.
	return function()
		-- Process events.
		if love.event then
			love.event.pump()
			for name, a,b,c,d,e,f in love.event.poll() do
				if name == "quit" then
					if not love.quit or not love.quit() then
						return a or 0
					end
				end
				xpcall(function() love.handlers[name](a,b,c,d,e,f) end, handle_error)
			end
		end

		-- Update dt, as we'll be passing it to update
		if love.timer then
			dt = love.timer.step()
		end

		-- Call update and draw
		if love.update then
			xpcall(function() love.update(dt) end, handle_error)
		end -- will pass 0 if love.timer is disabled

		if love.graphics and love.graphics.isActive() then
			love.graphics.origin()
			love.graphics.clear(love.graphics.getBackgroundColor())
			if love.draw then
				xpcall(love.draw, handle_error)
			end
			love.graphics.present()
		end

		if love.timer then
			love.timer.sleep(0.001)
		end
	end
end

-- Don't set the background color in this file. It's unlikely to have effect,
-- or might have surprising results.
-- See the comment on the call to setBackgroundColor in console.lua.

function love.load()
	love.keyboard.setKeyRepeat(true)
	love.keyboard.setTextInput(true)

	W, H, Flags = love.window.getMode()
	EditorY = MenuY + MenuHeight + console.FONT:getHeight() + console.VERTICAL_MARGIN

	-- Try to position and size window to stay above a popup soft keyboard on a fairly small tablet.
	-- Larger devices will have to resize it manually. (But it should remember its size after the first time.)
	EditorH = love.graphics.getHeight()-300
	if OS == 'iOS' then
		EditorH = 300  -- tested with iPad at 1194x834
	elseif OS == 'Android' then
		EditorH = EditorH-100  -- tested with Galaxy Tab A7 Lite
	end

	-- global directory to save and read files from (iOS = sourceBaseDirectory = "Documents")
	SourceBaseDir = love.filesystem.getSourceBaseDirectory()

	-- global variables for Slab
	InputText = "Write Code Here"

	-- Slab.Initialize does many things to include changing Love Keyboard handler to reference its own OnKeyPressed() & Love textinput handler to reference its own callback TextInput() and calls LoadState() [saved Slab State];
	-- In addition, uses function (not callback) Keyboard.isPressed(key) instead of Love keyboard.isDown(key)
	-- Slab.IsKeyPressed(key) calls Keyboard.isPressed(key)
	Slab.Initialize(--[[args]] nil, --[[dontInterceptEventHandlers]] true)
	LineHeight = Slab.GetTextHeight()
	SyntaxHighlight = {["and"]={0,0,1},["end"]={0,0,1},["in"]={0,0,1},["repeat"]={0,0,1},["break"]={0,0,1},["false"]={0,0,1},["local"]={0,0,1},["return"]={0,0,1},["do"]={0,0,1},["for"]={0,0,1},["nil"]={0,0,1},["then"]={0,0,1},["else"]={0,0,1},["function"]={0,0,1},["not"]={0,0,1},["true"]={0,0,1},["elseif"]={0,0,1},["if"]={0,0,1},["or"]={0,0,1},["until"]={0,0,1},["while"]={0,0,1}}

	-- save all MiniIDE handlers
	MiniIDE = {}
	MiniIDE.draw = love.draw
	MiniIDE.update = love.update
	MiniIDE_handlers = {}
	for name,_ in pairs(love.handlers) do
		MiniIDE_handlers[name] = love[name]
	end
end

function handle_error(err)
	Error_message = debug.traceback('Error: ' .. tostring(err), --[[stack frame]]2):gsub('\n[^\n]+$', '')
	print(Error_message)  -- save error to console
	-- reset MiniIDE state
	Slab.ResetFrame()
	-- restore all MiniIDE handlers
	love.draw = MiniIDE.draw
	love.update = MiniIDE.update
	for name,def in pairs(MiniIDE_handlers) do
		love[name] = def
	end
end

function love.keypressed(key, scancode, isrepeat)
	love.keyboard.setTextInput(true)  -- magic. keeps iOS from losing textinput events after switching apps.
	Slab.OnKeyPressed(key, scancode, isrepeat)
	if not(Slab.IsAnyInputFocused()) then console.keypressed(key, scancode, isrepeat) end
end

function love.textinput(text)
	Slab.OnTextInput(text)
	if not(Slab.IsAnyInputFocused()) then console.textinput(text) end
end

function love.keyreleased(key, scancode)
	Slab.OnKeyReleased(key, scancode)
	if not(Slab.IsAnyInputFocused()) then console.keyreleased(key, scancode) end
end

function love.update(dt)
	Slab.Update(dt)

	if Slab.BeginMainMenuBar(0.0, MenuY) then
		if Slab.BeginMenu("Console") then
			if Slab.MenuItem("Increase font size") then
				console.FONT_SIZE = console.FONT_SIZE + 1
				console.FONT = love.graphics.newFont(console.FONT_SIZE)
			end
			if Slab.MenuItem("Decrease font size") then
				console.FONT_SIZE = console.FONT_SIZE - 1
				console.FONT = love.graphics.newFont(console.FONT_SIZE)
			end
			if Slab.MenuItem("Clear") then
				addText("clear")
			end
			Slab.EndMenu()
		end
		if Slab.BeginMenu("Editor") then
			if Slab.MenuItem("Load") then
				Slab.OpenDialog("load dialog")
			end
			if Slab.MenuItem("Save") then
				Slab.OpenDialog("save dialog")
			end
			if Slab.MenuItem("Run") then
				addText(InputText)
			end
			if Slab.MenuItem("Copy highlight") then
				love.system.setClipboardText(Slab.GetSelection("EditorWindow.EditBox"))
			end
			if Slab.MenuItem("Paste at cursor") then
				InputText = Slab.InsertText("EditorWindow.EditBox", love.system.getClipboardText())
			end
			if Slab.MenuItem("Clear") then
				InputText = ""
			end
			if Slab.MenuItem("Append function to return to MiniIDE") then
				InputText = InputText.."\n".."-- call this from a button or hotkey to return to MiniIDE\nfunction escape()".."\n\t".."error('StopScript')".."\n".."end"
			end
			Slab.EndMenu()
		end
		-- Android doesn't autodetect orientation,
		-- so provide an explicit option to toggle between landscape and portrait mode.
		if OS == 'Android' then
			if Slab.BeginMenu("Rotate device") then
				if Slab.MenuItem("Rotate device") then
					W, H = H, W
					love.window.setMode(W, H, Flags)
				end
				Slab.EndMenu()
			end
		end
		Slab.EndMainMenuBar()
	end

	Slab.BeginWindow("EditorWindow",{Title = "Editor", AutoSizeWindow = false, X=love.graphics.getWidth()/2, Y = EditorY, W=love.graphics.getWidth()/2, H = EditorH, IsOpen = true})
	local W,H = Slab.GetWindowActiveSize()

	Slab.DisableDocks({"Left","Right","Bottom"})

	xpcall(function()

		--  Complicated Edit Box Setup: size in refernce to whole Window; Scale down height to
		--  Account for below edit box controls; have multiline wrap -10 from right edge
		if Slab.Input('EditBox',{Text = InputText, MultiLine = true, MultiLineW = W-10, W=W, H=H, Highlight = SyntaxHighlight}) then
			InputText = Slab.GetInputText()
		end

	end,
	function(err)  -- on error
		print('could not load '..LoadFilename..': '..err)
		InputText = ''  -- eliminate any confusion regarding which file we're editing
	end)

	if Slab.BeginDialog("load dialog", {Title = "Load File"}) then
		Slab.Text("Load Filename: ")
		Slab.SameLine()
		if Slab.BeginComboBox('filePicker',{Selected = LoadFilename}) then
			for index,info in ipairs(nativefs.getDirectoryItemsInfo(SourceBaseDir)) do
				if info.type == 'file' then
					if Slab.TextSelectable(info.name) then
						LoadFilename = info.name
					end
				end
			end
			Slab.EndComboBox()
		end
		Slab.NewLine()
		if Slab.Button("Load") and LoadFilename then
			InputText = nativefs.read(SourceBaseDir.."/"..LoadFilename)
			Slab.CloseDialog()
		end
		Slab.SameLine()
		if Slab.Button("Cancel") then
			Slab.CloseDialog()
		end
		Slab.EndDialog()
	end

	if Slab.BeginDialog("save dialog", {Title = "Save File"}) then
		Slab.Text("Save Filename: ")
		Slab.SameLine()
		if Slab.Input("writeFileName",{Text=SaveFilename}) then
			SaveFilename = Slab.GetInputText()
		end
		Slab.NewLine()
		if Slab.Button("Save") and SaveFilename then
			nativefs.write(SourceBaseDir.."/"..SaveFilename, InputText)
			print("file: "..SaveFilename.." written")
			Slab.CloseDialog()
		end
		Slab.SameLine()
		if Slab.Button("Cancel") then
			Slab.CloseDialog()
		end
		Slab.EndDialog()
	end

	Slab.EndWindow()
end

function love.draw()
	console.draw()  -- draws all user code if it doesn't override love.draw
	Slab.Draw()
end

function addText(text)
	console.addHistory(text)
	console.execute(text)
end

function love.quit()
	Slab.OnQuit()
end

function love.wheelmoved(x, y)
	Slab.OnWheelMoved(x, y)
end

function love.mousemoved(x, y, dx, dy, istouch)
	Slab.OnMouseMoved(x, y, dx, dy, istouch)
end

function love.mousepressed( x, y, button, istouch, presses)
	Slab.OnMousePressed( x, y, button, istouch, presses)
	if y > MenuY and y < MenuY + MenuHeight + console.FONT:getHeight() + console.VERTICAL_MARGIN then
		love.keyboard.setTextInput(true)
	end
end

function love.mousereleased( x, y, button, istouch, presses)
	Slab.OnMouseReleased( x, y, button, istouch, presses)
end
